@@ -28,32 +28,38 @@ pub struct Request {
     /// specified authors. at most 20s.
     uid: Option<Vec<u32>>,
     /// Not very convenient. you should consider use tags instead.
-    keyword: Option<String>,
+    keyword: Keyword,
     /// at most 20s
     tag: Tag,
     /// available values were defined in its setter.
     size: Size,
     /// proxy for `pixiv.net`, `i.pixiv.cat`, e.g. See [Lolicon](https://api.lolicon.app/#/setu?id=proxy) for detail.
-    proxy: Option<String>,
+    proxy: Proxy,
     /// Only show artworks after this UNIX time in millisecond.
-    date_after: DataAfter,
+    date_after: DateAfter,
     /// Only show artworks before this UNIX time in millisecond.
-    date_before: DataBefore,
+    date_before: DateBefore,
     /// If this is `true`, some automatic convert between keywords and tags will be disabled.
     dsc: Option<bool>,
 }
 
 #[derive(Debug, Clone)]
-struct Tag(Option<Vec<String>>);
+pub struct Keyword(Option<String>);
 
 #[derive(Debug, Clone)]
-struct Size(Option<Vec<String>>);
+pub struct Tag(Option<Vec<String>>);
 
 #[derive(Debug, Clone)]
-struct DataAfter(Option<u64>);
+pub struct Size(Option<Vec<String>>);
 
 #[derive(Debug, Clone)]
-struct DataBefore(Option<u64>);
+pub struct Proxy(Option<String>);
+
+#[derive(Debug, Clone)]
+pub struct DateAfter(Option<u64>);
+
+#[derive(Debug, Clone)]
+pub struct DateBefore(Option<u64>);
 
 #[derive(Copy, Clone, Debug)]
 pub enum R18 {
@@ -68,23 +74,25 @@ impl std::default::Default for Request {
             r18: None,
             num: None,
             uid: None,
-            keyword: None,
+            keyword: Keyword(None),
             tag: Tag(None),
             size: Size(None),
-            proxy: None,
-            date_after: DataAfter(None),
-            date_before: DataBefore(None),
+            proxy: Proxy(None),
+            date_after: DateAfter(None),
+            date_before: DateBefore(None),
             dsc: None,
         }
     }
 }
 
 impl Request {
+    /// set whether the result includes R18 artworks.
     pub fn r18(mut self, r: R18) -> Self {
         self.r18 = Some(r);
         self
     }
 
+    /// set amount of result's artworks.
     pub fn num(mut self, amount: u8) -> Result<Self, LoliError> {
         match amount {
             1..=100 => {
@@ -95,6 +103,7 @@ impl Request {
         }
     }
 
+    /// set artworks' authors.
     pub fn uid(mut self, authors: Vec<u32>) -> Result<Self, LoliError> {
         match authors.len() {
             1..=20 => {
@@ -105,11 +114,13 @@ impl Request {
         }
     }
 
-    pub fn keyword(mut self, keyword: String) -> Self {
-        self.keyword = Some(keyword);
+    /// set keyword.
+    pub fn keyword(mut self, keyword: impl Into<String>) -> Self {
+        self.keyword.0 = Some(keyword.into());
         self
     }
 
+    /// set tags.
     pub fn tag(mut self, tag: Vec<String>) -> Result<Self, LoliError> {
         match tag.len() {
             1..=20 => {
@@ -120,6 +131,7 @@ impl Request {
         }
     }
 
+    /// set sizes. `original`, `regular`, `small`, `thumb`, `mini` are available.
     pub fn size(mut self, size_list: Vec<String>) -> Result<Self, LoliError> {
         let sizes = ["original", "regular", "small", "thumb", "mini"];
         match size_list.len() {
@@ -135,17 +147,38 @@ impl Request {
             _ => Err(LoliError::IllegalSize),
         }
     }
+
+    pub fn proxy(mut self, proxy: impl Into<String>) -> Self {
+        self.proxy.0 = Some(proxy.into());
+        self
+    }
 }
 
 impl From<Request> for String {
     fn from(req: Request) -> Self {
         let mut url: String = "https://api.lolicon.app/setu/v2?".into();
-        req.r18.argument(&mut url);
-        req.num.argument(&mut url);
-        req.uid.argument(&mut url);
-        req.keyword.argument(&mut url);
-        req.tag.argument(&mut url);
-        req.size.argument(&mut url);
+
+        url.add_argument(req.r18);
+        url.add_argument(req.num);
+        url.add_argument(req.uid);
+        url.add_argument(req.keyword);
+        url.add_argument(req.tag);
+        url.add_argument(req.size);
+        url.add_argument(req.proxy);
+        url.add_argument(req.date_after);
+        url.add_argument(req.date_before);
+        url.add_argument(req.dsc);
+
         url
     }
-}
\ No newline at end of file
+}
+
+trait AddArgument {
+    fn add_argument(&mut self, object: impl Argument);
+}
+
+impl AddArgument for String {
+    fn add_argument(&mut self, object: impl Argument) {
+        object.argument(self);
+    }
+}
